﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Windows.Forms;
using HeuristicLab.Collections;
using HeuristicLab.Core;
using HeuristicLab.Core.Views;
using HeuristicLab.Data;
using HeuristicLab.DataPreprocessing.Filter;
using HeuristicLab.MainForm;

namespace HeuristicLab.DataPreprocessing.Views {
  [View("CheckedFilterCollection View")]
  [Content(typeof(FilterContent), true)]
  public partial class FilterView : ItemView {
    private CheckedItemCollectionView<IFilter> filterView;

    public FilterView() {
      InitializeComponent();
      tbTotal.Text = "0";
      tbFiltered.Text = "0";
      tbPercentage.Text = "0%";
      filterView = new CheckedItemCollectionView<IFilter>();
      filterView.Dock = DockStyle.Fill;
      filterView.Content = new CheckedItemCollection<IFilter>();
      filterView.Content.ItemsAdded += Content_ItemsAdded;
      filterView.Content.ItemsRemoved += Content_ItemsRemoved;
      filterView.Content.CheckedItemsChanged += Content_CheckedItemsChanged;
      groupBoxFilter.Controls.Add(filterView);
    }

    public new FilterContent Content {
      get { return (FilterContent)base.Content; }
      set { base.Content = value; }
    }

    private void Content_CheckedItemsChanged(object sender, Collections.CollectionItemsChangedEventArgs<IFilter> e) {

      foreach (IFilter filter in e.Items) {
        if (filterView.Content.CheckedItems.Contains(filter) && (filter.ConstraintData == null || string.IsNullOrEmpty(((StringValue)filter.ConstraintData).Value))) {
          filter.ConstraintData = new StringValue("0");
        }
        filter.Active = !filter.Active;
      }
      UpdateFilterInfo();
      Refresh();
      ResumeRepaint(true);
    }

    private void UpdateFilterInfo() {
      List<IFilter> filters = filterView.ItemCollection.ToList<IFilter>();

      int activeFilters = filters.Count(c => c.Active);

      applyFilterButton.Enabled = (activeFilters > 0);

      bool[] result = Content.FilterLogic.Preview(filters);

      int filteredCnt = result.Count(c => !c);

      tbFiltered.Text = filteredCnt.ToString();
      double percentage = filteredCnt * 100 / result.Length;
      tbPercentage.Text = percentage.ToString() + "%";
      tbTotal.Text = result.Length.ToString();
    }

    private void applyFilterButton_Click(object sender, EventArgs e) {
      List<IFilter> filters = filterView.ItemCollection.ToList<IFilter>();
      Content.FilterLogic.Apply(filters);
    }

    //whenever a new filter is added the preprocessing data is set to the filter
    private void Content_ItemsAdded(object sender, Collections.CollectionItemsChangedEventArgs<IFilter> e) {
      foreach (IFilter filter in e.Items) {
        filter.ConstrainedValue = Content.FilterLogic.PreprocessingData;
        filter.Active = true;
        filterView.Content.SetItemCheckedState(filter, false);
      }
      UpdateFilterInfo();
    }

    private void Content_ItemsRemoved(object sender, CollectionItemsChangedEventArgs<IFilter> e) {
      UpdateFilterInfo();
    }

  }
}
